#! /bin/bash

# Script to parse a text devinfo file, and write the output to a
# scsi kernel special file.
#
# The devinfo input file is of the form:
#
# 	vendor:product:{flag}[|flag]...
#
# Where the "flag" values are one of the BLIST_ below.
# 
# No error checking is done on the input file.
#

# The following values are taken from the kernel include/scsi/scsi_devinfo.h

BLIST_NOLUN=0x001
BLIST_FORCELUN=0x002
BLIST_BORKEN=0x004
BLIST_KEY=0x008
BLIST_SINGLELUN=0x010
BLIST_NOTQ=0x020
BLIST_SPARSELUN=0x040
BLIST_MAX5LUN=0x080
BLIST_ISROM=0x100
BLIST_LARGELUN=0x200
BLIST_INQUIRY_36=0x400
BLIST_INQUIRY_58=0x800
BLIST_NOSTARTONADD=0x1000
BLIST_MS_SKIP_PAGE_08=0x2000
BLIST_MS_SKIP_PAGE_3F=0x4000
BLIST_USE_10_BYTE_MS=0x8000
BLIST_MS_192_BYTES_FOR_3F=0x10000
BLIST_REPORTLUN2=0x20000
BLIST_NOREPORTLUN=0x40000
BLIST_NOT_LOCKABLE=0x80000
BLIST_NO_ULD_ATTACH=0x100000
BLIST_SELECT_NO_ATN=0x200000

devinfo_files=/etc/scsi/devinfo-*
devinfo_files=./devinfo-addon
devinfo_out=/proc/scsi/device_info

setup_flags()
{
	flags=
	for i in $*
	do
		if [ ${i} != "|" ]
		then
			if [ "${flags}" = "" ]
			then
				flags=${!i}
			else
				flags="${flags}|${!i}"
			fi
		fi
	done
	flags=$(printf "0x%x" $((flags)))
}

dump_one()
{
	if [ "${line}" = "" ]
	then
		return
	elif [ ${line:0:1} = "#" ]
	then
		# Skip comment lines, they have to start with #, unlike
		# these lines.
		return
	fi
	vendor_product=${line%:*}
	bflags=${line##*:}
	setup_flags ${bflags}
	echo \"${vendor_product}:${flags}\" > ${devinfo_out}
}

cat ${devinfo_files} | while read line
do
	dump_one ${line}
done
